{******************************************************}
{  PrnInfo Unit for Borland Delphi 5                   }
{  Copyright  2002 by Dimka Maslov                    }
{                                                      }
{  This unit allows to obtain information about        }
{  installed printers. See comments below.             }
{******************************************************}


unit PrnInfo;

interface

uses Windows, SysUtils, Classes, WinSpool, Dim;

type
  EPrinterInfo = class (Exception);

  { TPrinterInfo class collects information about installed printers and
    provides properties to return obtained data }
  TPrinterInfo = class (TObject)
  private
    FPtr: Pointer;
    FPI2: PPrinterInfo2;
    FNeeded, FCopied: DWORD;
    procedure Locate(PrinterIndex: Integer);
    function GetPaperFormat(PrinterIndex: Integer): TString;
    function GetPaperSize(PrinterIndex: Integer): TString;
    function GetXResolution(PrinterIndex: Integer): Integer;
    function GetYResolution(PrinterIndex: Integer): Integer;
    function GetPrinterInfo2(PrinterIndex: Integer): TPrinterInfo2;
    function GetDevMode(PrinterIndex: Integer): TDevMode;
    function GetPrintersCount: Integer;
    function GetPrinterName(PrinterIndex: Integer): TString;
  public
  { The Update procedure reads information about installed printers. Call this
    method prior to using an object's properties }
    procedure Update;

  { The PrintersCount property contains the number of installed printers. }
    property PrintersCount: Integer read GetPrintersCount;

  { The PrinterName property contains the names of installed printers. }
    property PrinterName[PrinterIndex: Integer]: TString read GetPrinterName;

  { The PaperFormat property contains the current paper formats of printers. }
    property PaperFormat[PrinterIndex: Integer]: TString read GetPaperFormat;

  { The PaperSize property contains information about the current paper sizes
    of printers. The string returned by this property has the following format:
    <units> <width> <height>. The '<units>' substring may be 'in' (inches) or
    'mm' (millimeters). The starting 'e' sign specifies to the parer size of an
    envelope. }
    property PaperSize[PrinterIndex: Integer]: TString read GetPaperSize;

  { The XResolution property contains the current dpi x-resolutions of printers. }
    property XResolution[PrinterIndex: Integer]: Integer read GetXResolution;

  { The YResolution property contains the current dpi y-resolutions of printers. }
    property YResolution[PrinterIndex: Integer]: Integer read GetYResolution;

  { The PrinterInfo2 property contains the PRINTER_INFO_2 structures of printers. }
    property PrinterInfo2[PrinterIndex: Integer]: TPrinterInfo2 read GetPrinterInfo2;

  { The DevMode propert contains the DEVMODE structures of printers. }
    property DevMode[PrinterIndex: Integer]: TDevMode read GetDevMode;

   { The Create constructor creates a new instance of this class. If the DoUpdate
     optional parameter is TRUE, the Update method will be called after creating
     an object. }
    constructor Create(DoUpdate: Boolean = False);

    destructor Destroy; override;
  end;

implementation

uses DimConst;

const
 PaperFormats:array[0..67] of TIdentMapItem=(
  (Value:DMPAPER_LETTER      ; Name:'Letter 8.5 x 11 in'),
  (Value:DMPAPER_LETTERSMALL ; Name:'Letter Small 8.5 x 11 in'),
  (Value:DMPAPER_TABLOID     ; Name:'Tabloid 11 x 17 in'),
  (Value:DMPAPER_LEDGER      ; Name:'Ledger 17 x 11 in'),
  (Value:DMPAPER_LEGAL       ; Name:'Legal 8.5 x 14 in'),
  (Value:DMPAPER_STATEMENT   ; Name:'Statement 5.5 x 8.5 in'),
  (Value:DMPAPER_EXECUTIVE   ; Name:'Executive 7.25 x 10.5 in'),
  (Value:DMPAPER_A3          ; Name:'A3 297 x 420 mm'),
  (Value:DMPAPER_A4          ; Name:'A4 210 x 297 mm'),
  (Value:DMPAPER_A4SMALL     ; Name:'A4 Small 210 x 297 mm'),
  (Value:DMPAPER_A5          ; Name:'A5 148 x 210 mm'),
  (Value:DMPAPER_B4          ; Name:'B4 (JIS) 250 x 354 mm'),
  (Value:DMPAPER_B5          ; Name:'B5 (JIS) 182 x 257 mm'),
  (Value:DMPAPER_FOLIO       ; Name:'Folio 8 12 x 13 in'),
  (Value:DMPAPER_QUARTO      ; Name:'Quarto 215 x 275 mm'),
  (Value:DMPAPER_10X14       ; Name:'10x14 in'),
  (Value:DMPAPER_11X17       ; Name:'11x17 in'),
  (Value:DMPAPER_NOTE        ; Name:'Note 8.5 x 11 in'),
  (Value:DMPAPER_ENV_9       ; Name:'Envelope #9 3.375 x 8.375'),
  (Value:DMPAPER_ENV_10      ; Name:'Envelope #10 4.125 x 9.5 in'),
  (Value:DMPAPER_ENV_11      ; Name:'Envelope #11 4.5 x 10.375 in'),
  (Value:DMPAPER_ENV_12      ; Name:'Envelope #12 4.75 x 11 in'),
  (Value:DMPAPER_ENV_14      ; Name:'Envelope #14 5 x 11.5 in'),
  (Value:DMPAPER_CSHEET      ; Name:'C size sheet 17 x 22 in'),
  (Value:DMPAPER_DSHEET      ; Name:'D size sheet 22 x 34 in'),
  (Value:DMPAPER_ESHEET      ; Name:'E size sheet 34 x 44 in'),
  (Value:DMPAPER_ENV_DL      ; Name:'Envelope DL 110 x 220 mm'),
  (Value:DMPAPER_ENV_C5      ; Name:'Envelope C5 162 x 229 mm'),
  (Value:DMPAPER_ENV_C3      ; Name:'Envelope C3  324 x 458 mm'),
  (Value:DMPAPER_ENV_C4      ; Name:'Envelope C4  229 x 324 mm'),
  (Value:DMPAPER_ENV_C6      ; Name:'Envelope C6  114 x 162 mm'),
  (Value:DMPAPER_ENV_C65     ; Name:'Envelope C65 114 x 229 mm'),
  (Value:DMPAPER_ENV_B4      ; Name:'Envelope B4  250 x 353 mm'),
  (Value:DMPAPER_ENV_B5      ; Name:'Envelope B5  176 x 250 mm'),
  (Value:DMPAPER_ENV_B6      ; Name:'Envelope B6  176 x 125 mm'),
  (Value:DMPAPER_ENV_ITALY   ; Name:'Envelope 110 x 230 mm'),
  (Value:DMPAPER_ENV_MONARCH ; Name:'Envelope Monarch 3.875 x 7.5 in'),
  (Value:DMPAPER_ENV_PERSONAL       ; Name:'6.75 Envelope 3.625 x 6.5 in'),
  (Value:DMPAPER_FANFOLD_US         ; Name:'US Std Fanfold 14.875 x 11 in'),
  (Value:DMPAPER_FANFOLD_STD_GERMAN ; Name:'German Std Fanfold 8.5 x 12 in'),
  (Value:DMPAPER_FANFOLD_LGL_GERMAN ; Name:'German Legal Fanfold 8.5 x 13 in'),
  (Value:DMPAPER_ISO_B4             ; Name:'B4 (ISO) 250 x 353 mm'),
  (Value:DMPAPER_JAPANESE_POSTCARD  ; Name:'Japanese Postcard 100 x 148 mm'),
  (Value:DMPAPER_9X11          ; Name:'9 x 11 in'),
  (Value:DMPAPER_10X11         ; Name:'10 x 11 in'),
  (Value:DMPAPER_15X11         ; Name:'15 x 11 in'),
  (Value:DMPAPER_ENV_INVITE    ; Name:'Envelope Invite 220 x 220 mm'),
  (Value:DMPAPER_LETTER_EXTRA  ; Name:'Letter Extra 9.5 x 12 in'),
  (Value:DMPAPER_LEGAL_EXTRA   ; Name:'Legal Extra 9.5 x 15 in'),
  (Value:DMPAPER_TABLOID_EXTRA ; Name:'Tabloid Extra 11.69 x 18 in'),
  (Value:DMPAPER_A4_EXTRA      ; Name:'A4 Extra 9.27 x 12.69 in'),
  (Value:DMPAPER_LETTER_TRANSVERSE       ; Name:'Letter Transverse 8.5 x 11 in'),
  (Value:DMPAPER_A4_TRANSVERSE           ; Name:'A4 Transverse 210 x 297 mm'),
  (Value:DMPAPER_LETTER_EXTRA_TRANSVERSE ; Name:'Letter Extra Transverse 9.5 x 12 in'),
  (Value:DMPAPER_A_PLUS        ; Name:'SuperASuperAA4 227 x 356 mm'),
  (Value:DMPAPER_B_PLUS        ; Name:'SuperBSuperBA3 305 x 487 mm'),
  (Value:DMPAPER_LETTER_PLUS   ; Name:'Letter Plus 8.5 x 12.69 in'),
  (Value:DMPAPER_A4_PLUS       ; Name:'A4 Plus 210 x 330 mm'),
  (Value:DMPAPER_A5_TRANSVERSE ; Name:'A5 Transverse 148 x 210 mm'),
  (Value:DMPAPER_B5_TRANSVERSE ; Name:'B5 (JIS) Transverse 182 x 257 mm'),
  (Value:DMPAPER_A3_EXTRA      ; Name:'A3 Extra 322 x 445 mm'),
  (Value:DMPAPER_A5_EXTRA      ; Name:'A5 Extra 174 x 235 mm'),
  (Value:DMPAPER_B5_EXTRA      ; Name:'B5 (ISO) Extra 201 x 276 mm'),
  (Value:DMPAPER_A2            ; Name:'A2 420 x 594 mm'),
  (Value:DMPAPER_A3_TRANSVERSE ; Name:'A3 Transverse 297 x 420 mm'),
  (Value:DMPAPER_A3_EXTRA_TRANSVERSE   ; Name:'A3 Extra Transverse 322 x 445 mm'),
  (Value:DMPAPER_DBL_JAPANESE_POSTCARD ; Name:'Japanese Double Postcard 200 x 148 mm'),
  (Value:DMPAPER_A6             ; Name:'A6 105 x 148 mm'));

 PaperSizes:array[0..67] of TIdentMapItem=(
  (Value:DMPAPER_LETTER      ; Name:'in 8.5 11'),
  (Value:DMPAPER_LETTERSMALL ; Name:'in 8.5 1'),
  (Value:DMPAPER_TABLOID     ; Name:'in 11 17'),
  (Value:DMPAPER_LEDGER      ; Name:'in 17 11'),
  (Value:DMPAPER_LEGAL       ; Name:'in 8.5 14'),
  (Value:DMPAPER_STATEMENT   ; Name:'in 5.5 8.5'),
  (Value:DMPAPER_EXECUTIVE   ; Name:'in 7.25 10.5'),
  (Value:DMPAPER_A3          ; Name:'mm 297 420'),
  (Value:DMPAPER_A4          ; Name:'mm 210 297'),
  (Value:DMPAPER_A4SMALL     ; Name:'mm 210 297'),
  (Value:DMPAPER_A5          ; Name:'mm 148 210'),
  (Value:DMPAPER_B4          ; Name:'mm 250 354'),
  (Value:DMPAPER_B5          ; Name:'mm 182 257'),
  (Value:DMPAPER_FOLIO       ; Name:'in 8.5 13'),
  (Value:DMPAPER_QUARTO      ; Name:'mm 215 275'),
  (Value:DMPAPER_10X14       ; Name:'in 10 14'),
  (Value:DMPAPER_11X17       ; Name:'in 11 17'),
  (Value:DMPAPER_NOTE        ; Name:'in 8.5 11'),
  (Value:DMPAPER_ENV_9       ; Name:'ein 3.375 8.375'),
  (Value:DMPAPER_ENV_10      ; Name:'ein 4.125 9.5'),
  (Value:DMPAPER_ENV_11      ; Name:'ein 4.5 10.375'),
  (Value:DMPAPER_ENV_12      ; Name:'ein 4.75 11'),
  (Value:DMPAPER_ENV_14      ; Name:'ein 5 11.5'),
  (Value:DMPAPER_CSHEET      ; Name:'in 17 22'),
  (Value:DMPAPER_DSHEET      ; Name:'in 22 34'),
  (Value:DMPAPER_ESHEET      ; Name:'in 34 44'),
  (Value:DMPAPER_ENV_DL      ; Name:'emm 110 220'),
  (Value:DMPAPER_ENV_C5      ; Name:'emm 162 229'),
  (Value:DMPAPER_ENV_C3      ; Name:'emm 324 458'),
  (Value:DMPAPER_ENV_C4      ; Name:'emm 229 324'),
  (Value:DMPAPER_ENV_C6      ; Name:'emm 114 162'),
  (Value:DMPAPER_ENV_C65     ; Name:'emm 114 229'),
  (Value:DMPAPER_ENV_B4      ; Name:'emm 250 353'),
  (Value:DMPAPER_ENV_B5      ; Name:'emm 176 250'),
  (Value:DMPAPER_ENV_B6      ; Name:'emm 176 125'),
  (Value:DMPAPER_ENV_ITALY   ; Name:'emm 110 230'),
  (Value:DMPAPER_ENV_MONARCH ; Name:'ein 3.875 7.5'),
  (Value:DMPAPER_ENV_PERSONAL       ; Name:'ein 3.625 6.5'),
  (Value:DMPAPER_FANFOLD_US         ; Name:'in 14.875 11'),
  (Value:DMPAPER_FANFOLD_STD_GERMAN ; Name:'in 8.5 12'),
  (Value:DMPAPER_FANFOLD_LGL_GERMAN ; Name:'in 8.5 13'),
  (Value:DMPAPER_ISO_B4             ; Name:'mm 250 353'),
  (Value:DMPAPER_JAPANESE_POSTCARD  ; Name:'mm 100 148'),
  (Value:DMPAPER_9X11          ; Name:'in 9 11'),
  (Value:DMPAPER_10X11         ; Name:'in 10 11'),
  (Value:DMPAPER_15X11         ; Name:'in 15 11'),
  (Value:DMPAPER_ENV_INVITE    ; Name:'emm 220 220'),
  (Value:DMPAPER_LETTER_EXTRA  ; Name:'in 9.5 12'),
  (Value:DMPAPER_LEGAL_EXTRA   ; Name:'in 9.5 15'),
  (Value:DMPAPER_TABLOID_EXTRA ; Name:'in 11.69 18'),
  (Value:DMPAPER_A4_EXTRA      ; Name:'in 9.27 12.69'),
  (Value:DMPAPER_LETTER_TRANSVERSE       ; Name:'in 8.5 11'),
  (Value:DMPAPER_A4_TRANSVERSE           ; Name:'mm 210 297'),
  (Value:DMPAPER_LETTER_EXTRA_TRANSVERSE ; Name:'in 9.5 12'),
  (Value:DMPAPER_A_PLUS        ; Name:'mm 227 356'),
  (Value:DMPAPER_B_PLUS        ; Name:'mm 305 487'),
  (Value:DMPAPER_LETTER_PLUS   ; Name:'in 8.5 12.69'),
  (Value:DMPAPER_A4_PLUS       ; Name:'mm 210 330'),
  (Value:DMPAPER_A5_TRANSVERSE ; Name:'mm 148 210'),
  (Value:DMPAPER_B5_TRANSVERSE ; Name:'mm 182 257'),
  (Value:DMPAPER_A3_EXTRA      ; Name:'mm 322 445'),
  (Value:DMPAPER_A5_EXTRA      ; Name:'mm 174 235'),
  (Value:DMPAPER_B5_EXTRA      ; Name:'mm 201 276'),
  (Value:DMPAPER_A2            ; Name:'mm 420 594'),
  (Value:DMPAPER_A3_TRANSVERSE ; Name:'mm 297 420'),
  (Value:DMPAPER_A3_EXTRA_TRANSVERSE   ; Name:'mm 322 445'),
  (Value:DMPAPER_DBL_JAPANESE_POSTCARD ; Name:'mm 200 148'),
  (Value:DMPAPER_A6             ; Name:'mm 105 148'));

{ TPrinterInfo }

constructor TPrinterInfo.Create(DoUpdate: Boolean);
begin
 inherited Create;
 FPtr:=nil;
 if DoUpdate then Update;
end;

destructor TPrinterInfo.Destroy;
begin
 if Assigned(FPtr) then DeallocateMem(FPtr);
 inherited;
end;

function TPrinterInfo.GetDevMode(PrinterIndex: Integer): TDevMode;
begin
 Locate(PrinterIndex);
 Result:=FPI2.pDevMode^;
end;

function TPrinterInfo.GetPaperFormat(PrinterIndex: Integer): TString;
begin
 Locate(PrinterIndex);
 Result:=ValueToName(FPI2.pDevMode.dmPaperSize, PaperFormats);
end;

function TPrinterInfo.GetPaperSize(PrinterIndex: Integer): TString;
begin
 Locate(PrinterIndex);
 Result:=ValueToName(FPI2.pDevMode.dmPaperSize, PaperSizes);
end;

function TPrinterInfo.GetPrinterInfo2(
  PrinterIndex: Integer): TPrinterInfo2;
begin
 Locate(PrinterIndex);
 Result:=FPI2^;
end;

function TPrinterInfo.GetPrinterName(PrinterIndex: Integer): TString;
begin
 Locate(PrinterIndex);
 Result:=FPI2.pPrinterName;
end;

function TPrinterInfo.GetPrintersCount: Integer;
begin
 Result:=FCopied;
end;

function TPrinterInfo.GetXResolution(PrinterIndex: Integer): Integer;
begin
 Locate(PrinterIndex);
 with FPI2.pDevMode^ do if dmPrintQuality>0 then Result:=dmPrintQuality
                                           else Result:=dmYResolution;
end;

function TPrinterInfo.GetYResolution(PrinterIndex: Integer): Integer;
begin
 Locate(PrinterIndex);
 Result:=FPI2.pDevMode.dmYResolution;
end;

procedure TPrinterInfo.Locate(PrinterIndex: Integer);
begin
 if not Inside(PrinterIndex, 0, Integer(FCopied)-1) then
  raise EPrinterInfo.CreateFmt(SPrinterIndexError, [PrinterIndex]);
 FPI2:=IncPtr(FPtr, PrinterIndex*SizeOf(TPrinterInfo2));
end;

procedure TPrinterInfo.Update;
begin
 if Assigned(FPtr) then begin
  DeallocateMem(FPtr);
  FNeeded:=0;
  FCopied:=0;
  FPtr:=nil;
 end;
 EnumPrinters(PRINTER_ENUM_LOCAL, nil, 2, FPtr, 0, FNeeded, FCopied);
 FPtr:=AllocateMem(FNeeded);
 EnumPrinters(PRINTER_ENUM_LOCAL, nil, 2, FPtr, FNeeded, FNeeded, FCopied);
end;

end.
